@@ -200,7 +200,7 @@ lws_role_call_adoption_bind(struct lws *wsi, int type, const char *prot)
 
 #if defined(LWS_ROLE_RAW_FILE)
 
-	lwsl_wsi_notice(wsi, "falling back to raw file role bind");
+	lwsl_wsi_info(wsi, "falling back to raw file role bind");
 
 	/* fall back to raw file role if, eg, h1 not configured */
 
@@ -305,7 +305,7 @@ lws_protocol_vh_priv_get(struct lws_vhost *vhost,
 		}
 
 		if (n == vhost->count_protocols) {
-			lwsl_vhost_err(vhost, "unknown protocol %p", prot);
+			lwsl_vhost_err(vhost, "unknown protocol %p (%s)", prot, prot->name);
 			return NULL;
 		}
 	}
@@ -390,7 +390,7 @@ lws_protocol_init_vhost(struct lws_vhost *vh, int *any)
 	struct lws _lws;
 	struct lws_a *lwsa = &_lws.a;
 
-	memset(&_lws, 0, sizeof(_lws));
+	memset((void *)&_lws, 0, sizeof(_lws));
 #endif
 
 	lwsa->context = vh->context;
@@ -402,6 +402,7 @@ lws_protocol_init_vhost(struct lws_vhost *vh, int *any)
 		lwsa->protocol = &vh->protocols[n];
 		if (!vh->protocols[n].name)
 			continue;
+
 		pvo = lws_vhost_protocol_options(vh, vh->protocols[n].name);
 		if (pvo) {
 			/*
@@ -450,7 +451,8 @@ lws_protocol_init_vhost(struct lws_vhost *vh, int *any)
 		 * prepared in case the protocol handler wants to touch them
 		 */
 
-		if (pvo
+		if (pvo || (vh->options & LWS_SERVER_OPTION_VH_INSTANTIATE_ALL_PROTOCOLS)
+
 #if !defined(LWS_WITH_PLUGINS)
 				/*
 				 * with plugins, you have to explicitly
@@ -466,22 +468,19 @@ lws_protocol_init_vhost(struct lws_vhost *vh, int *any)
 			lwsl_vhost_info(vh, "init %s.%s", vh->name,
 					vh->protocols[n].name);
 			if (vh->protocols[n].callback((struct lws *)lwsa,
-				LWS_CALLBACK_PROTOCOL_INIT, NULL,
-#if !defined(LWS_WITH_PLUGINS)
-				(void *)(pvo ? pvo->options : NULL),
-#else
-				(void *)pvo->options,
-#endif
-				0)) {
+					LWS_CALLBACK_PROTOCOL_INIT, NULL,
+					(void *)(pvo ? pvo->options : NULL),
+					0)) {
 				if (vh->protocol_vh_privs && vh->protocol_vh_privs[n]) {
 					lws_free(vh->protocol_vh_privs[n]);
 					vh->protocol_vh_privs[n] = NULL;
 				}
-			lwsl_vhost_err(vh, "protocol %s failed init",
+				lwsl_vhost_warn(vh, "protocol %s failed init",
 					vh->protocols[n].name);
 
-				return 1;
-			}
+				// return 1;
+			} else
+				vh->protocol_init |= 1u << n;
 		}
 	}
 
@@ -590,6 +589,9 @@ lws_create_vhost(struct lws_context *context,
 #endif
 	int n;
 
+	if (!pcols && context->protocols_copy)
+		pcols = context->protocols_copy;
+
 	if (info->vhost_name)
 		name = info->vhost_name;
 
@@ -683,12 +685,14 @@ lws_create_vhost(struct lws_context *context,
 	 * old or new way depending on what he gave us
 	 */
 
-	if (!pcols)
+	if (!pcols) {
 		for (vh->count_protocols = 0;
 			info->pprotocols[vh->count_protocols];
 			vh->count_protocols++)
-			;
-	else
+				;
+			//lwsl_user("%s: ppcols: %s\n", __func__,
+			// info->pprotocols[vh->count_protocols]->name);
+	} else
 		for (vh->count_protocols = 0;
 			pcols[vh->count_protocols].callback;
 			vh->count_protocols++)
@@ -741,6 +745,8 @@ lws_create_vhost(struct lws_context *context,
 	if (n) {
 		vh->tls.key_path = vh->tls.alloc_cert_path =
 					lws_malloc((unsigned int)n, "vh paths");
+		if (!vh->tls.alloc_cert_path)
+			goto bail;
 		if (info->ssl_cert_filepath) {
 			n = (int)strlen(info->ssl_cert_filepath) + 1;
 			memcpy(vh->tls.alloc_cert_path,
@@ -812,9 +818,20 @@ lws_create_vhost(struct lws_context *context,
 	 */
 #if defined(LWS_WITH_SYS_ASYNC_DNS)
 	if (!context->vhost_list) {
-		memcpy(&lwsp[m++], &lws_async_dns_protocol,
-		       sizeof(struct lws_protocols));
-		vh->count_protocols++;
+		uint8_t seen = 0;
+
+		for (n = 0; n < m; n++)
+			if (!memcmp(&lwsp[n], &lws_async_dns_protocol, sizeof(struct lws_protocols))) {
+				/* Already defined */
+				seen = 1;
+				break;
+			}
+
+		if (!seen) {
+			memcpy(&lwsp[m++], &lws_async_dns_protocol,
+			       sizeof(struct lws_protocols));
+			vh->count_protocols++;
+		}
 	}
 #endif
 
@@ -832,7 +849,8 @@ lws_create_vhost(struct lws_context *context,
 	 * for a protocol get it enabled.
 	 */
 
-	if (context->options & LWS_SERVER_OPTION_EXPLICIT_VHOSTS)
+	if ((context->options & LWS_SERVER_OPTION_EXPLICIT_VHOSTS) &&
+	    !(vh->options & LWS_SERVER_OPTION_VH_INSTANTIATE_ALL_PROTOCOLS))
 		f = 0;
 	(void)f;
 #ifdef LWS_WITH_PLUGINS
@@ -997,11 +1015,13 @@ lws_create_vhost(struct lws_context *context,
 		lwsl_vhost_err(vh, "lws_context_init_server_ssl failed");
 		goto bail1;
 	}
+#if defined(LWS_WITH_CLIENT)
 	if (lws_fi(&vh->fic, "vh_create_ssl_cli") ||
 	    lws_context_init_client_ssl(info, vh)) {
 		lwsl_vhost_err(vh, "lws_context_init_client_ssl failed");
 		goto bail1;
 	}
+#endif
 #if defined(LWS_WITH_SERVER)
 	lws_context_lock(context, __func__);
 	if (lws_fi(&vh->fic, "vh_create_srv_init"))
@@ -1059,6 +1079,14 @@ early_bail:
 	return NULL;
 }
 
+void
+lws_vhost_set_mounts(struct lws_vhost *vh, const struct lws_http_mount *mounts)
+{
+#if defined(LWS_ROLE_H1) || defined(LWS_ROLE_H2)
+        vh->http.mount_list = mounts;
+#endif
+}
+
 int
 lws_init_vhost_client_ssl(const struct lws_context_creation_info *info,
 			  struct lws_vhost *vhost)
@@ -1081,7 +1109,7 @@ void
 lws_cancel_service(struct lws_context *context)
 {
 	struct lws_context_per_thread *pt = &context->pt[0];
-	short m;
+	unsigned short m;
 
 	if (context->service_no_longer_possible)
 		return;
@@ -1144,6 +1172,7 @@ __lws_create_event_pipes(struct lws_context *context)
 
 			if (lws_wsi_inject_to_loop(pt, wsi))
 					goto bail;
+			return 0;
 		}
 	}
 
@@ -1209,8 +1238,7 @@ __lws_vhost_destroy_pt_wsi_dieback_start(struct lws_vhost *vh)
 		if (w->tsi == tsi) {
 
 			lwsl_vhost_debug(vh, "closing aso");
-			lws_close_free_wsi(w, LWS_CLOSE_STATUS_NOSTATUS,
-					   "awaiting skt");
+			lws_wsi_close(w, LWS_TO_KILL_ASYNC);
 		}
 
 	} lws_end_foreach_dll_safe(d, d1);
@@ -1455,7 +1483,7 @@ __lws_vhost_destroy2(struct lws_vhost *vh)
 	 * let the protocols destroy the per-vhost protocol objects
 	 */
 
-	memset(&wsi, 0, sizeof(wsi));
+	memset((void *)&wsi, 0, sizeof(wsi));
 	wsi.a.context = vh->context;
 	wsi.a.vhost = vh; /* not a real bound wsi */
 	protocol = vh->protocols;
@@ -1464,11 +1492,11 @@ __lws_vhost_destroy2(struct lws_vhost *vh)
 		while (n < vh->count_protocols) {
 			wsi.a.protocol = protocol;
 
-			lwsl_vhost_debug(vh, "protocol destroy");
-
-			if (protocol->callback)
+			if (protocol->callback && (vh->protocol_init & (1u << n))) {
+				lwsl_vhost_debug(vh, "protocol %s destroy", protocol->name);
 				protocol->callback(&wsi, LWS_CALLBACK_PROTOCOL_DESTROY,
 					   NULL, NULL, 0);
+			}
 			protocol++;
 			n++;
 		}
@@ -1548,6 +1576,7 @@ __lws_vhost_destroy2(struct lws_vhost *vh)
 
 #if defined (LWS_WITH_TLS)
 	lws_free_set_NULL(vh->tls.alloc_cert_path);
+	lws_free_set_NULL(vh->tls.key_path);
 #endif
 
 #if LWS_MAX_SMP > 1
@@ -1891,7 +1920,7 @@ lws_vhost_active_conns(struct lws *wsi, struct lws **nwsi, const char *adsin)
 			 * to get there or fail.
 			 */
 
-			lwsl_wsi_notice(wsi, "apply txn queue %s, state 0x%lx",
+			lwsl_wsi_info(wsi, "apply txn queue %s, state 0x%lx",
 					     lws_wsi_tag(w),
 					     (unsigned long)w->wsistate);
 			/*
