@@ -94,7 +94,7 @@ const struct http2_settings lws_h2_stock_settings = { {
  * another path via lws_service_do_ripe_rxflow() on mux children too tho...
  */
 
-static int
+static lws_handling_result_t
 rops_handle_POLLIN_h2(struct lws_context_per_thread *pt, struct lws *wsi,
 		       struct lws_pollfd *pollfd)
 {
@@ -376,7 +376,8 @@ drain:
 	return LWS_HPI_RET_HANDLED;
 }
 
-int rops_handle_POLLOUT_h2(struct lws *wsi)
+lws_handling_result_t
+rops_handle_POLLOUT_h2(struct lws *wsi)
 {
 	// lwsl_notice("%s\n", __func__);
 
@@ -738,7 +739,7 @@ rops_close_kill_connection_h2(struct lws *wsi, enum lws_close_status reason)
 
 		while (w) {
 			w1 = w->next;
-			free(w);
+			lws_free(w);
 			w = w1;
 		}
 		wsi->h2.h2n->pps = NULL;
@@ -865,6 +866,20 @@ lws_h2_bind_for_post_before_action(struct lws *wsi)
 
 		if (lws_bind_protocol(wsi, pp, __func__))
 			return 1;
+#if defined(LWS_WITH_HTTP_BASIC_AUTH)
+		/* basic auth? */
+
+		switch (lws_check_basic_auth(wsi, hit->basic_auth_login_file,
+					     hit->auth_mask & AUTH_MODE_MASK)) {
+		case LCBA_CONTINUE:
+			break;
+		case LCBA_FAILED_AUTH:
+			return lws_unauthorised_basic_auth(wsi);
+		case LCBA_END_TRANSACTION:
+			lws_return_http_status(wsi, HTTP_STATUS_FORBIDDEN, NULL);
+			return lws_http_transaction_completed(wsi);
+		}
+#endif
 	}
 
 	methidx = lws_http_get_uri_and_method(wsi, &uri_ptr, &uri_len);
