@@ -137,6 +137,10 @@ struct _lws_websocket_related {
 	uint32_t rx_ubuf_head;
 	uint32_t rx_ubuf_alloc;
 
+	uint8_t		last_valid;
+	uint8_t		last_opcode;	/* for outgoing state validation */
+	uint8_t		last_fin;	/* for outgoing state validation */
+
 	uint8_t pong_payload_len;
 	uint8_t mask_idx;
 	uint8_t opcode;
@@ -168,6 +172,9 @@ struct lws_ext_pm_deflate_rx_ebufs {
 lws_handling_result_t
 lws_ws_handshake_client(struct lws *wsi, unsigned char **buf, size_t len);
 
+lws_handling_result_t
+lws_ws_client_rx_parser_block(struct lws *wsi, const uint8_t **buf, size_t *len);
+
 #if !defined(LWS_WITHOUT_EXTENSIONS)
 LWS_VISIBLE void
 lws_context_init_extensions(const struct lws_context_creation_info *info,
