@@ -23,7 +23,7 @@
 /**
  * @brief create a certification object according to input certification
  */
-CERT *__ssl_cert_new(CERT *ic, void *rngctx)
+CERT *__ssl_cert_new(CERT *ic)
 {
     CERT *cert;
 
@@ -44,7 +44,7 @@ CERT *__ssl_cert_new(CERT *ic, void *rngctx)
         ix = NULL;
     }
 
-    cert->pkey = __EVP_PKEY_new(ipk, rngctx);
+    cert->pkey = __EVP_PKEY_new(ipk);
     if (!cert->pkey) {
         SSL_DEBUG(SSL_CERT_ERROR_LEVEL, "__EVP_PKEY_new() return NULL");
         goto pkey_err;
@@ -69,9 +69,9 @@ no_mem:
 /**
  * @brief create a certification object include private key object
  */
-CERT *ssl_cert_new(void *rngctx)
+CERT *ssl_cert_new(void)
 {
-    return __ssl_cert_new(NULL, rngctx);
+    return __ssl_cert_new(NULL);
 }
 
 /**
